@@ -62,7 +62,7 @@ screen_dump ()
 #define R_DISPOSE "print/delete %s"	/* printer dispose command */
 #define P_DISPOSE "print/delete/queue=%s %s"  /* dispose to user's printer */
 #else
-#define	R_DISPOSE "lpr -Plp -r %s"	/* printer dispose command	*/
+#define	R_DISPOSE "lpr -Plp60_1 -r %s"	/* printer dispose command	*/
 #define P_DISPOSE "lpr -P%s -r %s"	/* dispose to user's printer	*/
 #endif
 #define	SEGSIZE		16		/* output segment size (bytes)	*/
