@@ -13,7 +13,6 @@ static char SccsId[] = "%W%  %G%";
  *		makes no representations about the suitability of this
  *		software for any purpose.  It is provided "as is" without
  *		express or implied warranty.
- *		{n} <who> -- <does what> -- <when>
  */
 
 #include <stdio.h>
@@ -79,6 +78,8 @@ set_iraf_key_trigger ( state )
 #endif
 
 static int direction = 1;
+static double oldxpix = 0.0;
+static double oldypix = 0.0;
 
 /*  Subroutine:	key_response
  *  Purpose:	Respond to a key stroke
@@ -111,6 +112,8 @@ key_response()
   double xbuf, ybuf, xfile, yfile;
   double oldxpos, oldypos, newxpos, newypos, distance;
   char sysout0[16];
+  char wcstr[32];
+  int lwcstr = 32;
 
   if (iswcs(wcs) && (wcs->prjcode == WCS_CSC ||
       wcs->prjcode == WCS_QSC || wcs->prjcode == WCS_TSC))
@@ -494,10 +497,14 @@ key_response()
     case XK_H:
     case XK_h:
 	if (iswcs (wcs)) {
-	    if (wcs->degout)
+	    if (wcs->degout) {
 		wcs->degout = 0;
-	    else
+		wcs->ndec = 3;
+		}
+	    else {
 		wcs->degout = 1;
+		wcs->ndec = 6;
+		}
 	    }
 	magnify_disp (&control.event, 0, 1);
 	break;
@@ -505,10 +512,12 @@ key_response()
     /* Print cursor position to standard output */
     case XK_C:
     case XK_c:
-	print_cursor (&control.event);
+	print_cursor (&control.event, 1);
 	if (iswcs (wcs)) {
 	    oldxpos = wcs->xpos;
 	    oldypos = wcs->ypos;
+	    oldxpix = wcs->xpix;
+	    oldypix = wcs->ypix;
 	    }
 	break;
 
@@ -516,7 +525,7 @@ key_response()
     case XK_V:
     case XK_v:
 	if (iswcs (wcs)) {
-	    print_cursor (&control.event);
+	    print_cursor (&control.event, 1);
 	    newxpos = wcs->xpos;
 	    newypos = wcs->ypos;
 	    distance = wcsdist (oldxpos, oldypos, newxpos, newypos);
@@ -531,23 +540,31 @@ key_response()
     case XK_W:
     case XK_w:
 	if (iswcs (wcs)) {
-	    (void)get_cursor_pos (&control.event, &xbuf, &ybuf, &xfile, &yfile);
-	    wcscom (wcs, 0, img.filename, xfile, yfile);
+	    (void)get_cursor_pos (&control.event,&xbuf,&ybuf,&xfile,&yfile);
+	    pix2wcst (wcs, xfile, yfile, wcstr, lwcstr);
+	    if (oldxpix != 0.0 && oldypix != 0.0) {
+		wcscom (wcs, 0, img.filename, oldxpix, oldypix, wcstr);
+		oldxpix = 0.0;
+		oldypix = 0.0;
+		}
+	    else {
+		wcscom (wcs, 0, img.filename, xfile, yfile, wcstr);
+		}
 	    }
 	break;
 
     case XK_X:
     case XK_x:
-	print_center ();
+	print_center (&oldxpix, &oldypix);
 	break;
 
     /* Toggle between AIPS and WCSLIB WCS subroutines */
     case XK_Z:
 	if (iswcs (wcs)) {
-	    if (wcs->oldwcs == 0)
-		wcs->oldwcs = 1;
+	    if (wcs->wcsproj == WCS_BEST)
+		wcs->wcsproj = WCS_ALT;
 	    else
-		wcs->oldwcs = 0;
+		wcs->wcsproj = WCS_BEST;
 	    }
 	magnify_disp (&control.event, 0, 1);
 	break;
@@ -558,49 +575,67 @@ key_response()
 
     /* Use WCS and image coordinates of cursor with an external command */
     case XK_F1:
-	if (iswcs (wcs))
+	if (iswcs (wcs)) {
 	    (void)get_cursor_pos (&control.event, &xbuf, &ybuf, &xfile, &yfile);
-	    wcscom (wcs, 1, img.filename, xfile, yfile);
+	    pix2wcst (wcs, xfile, yfile, wcstr, lwcstr);
+	    wcscom (wcs, 1, img.filename, xfile, yfile, wcstr);
+	    }
 	break;
     case XK_F2:
-	if (iswcs (wcs))
+	if (iswcs (wcs)) {
 	    (void)get_cursor_pos (&control.event, &xbuf, &ybuf, &xfile, &yfile);
-	    wcscom (wcs, 2, img.filename, xfile, yfile);
+	    pix2wcst (wcs, xfile, yfile, wcstr, lwcstr);
+	    wcscom (wcs, 2, img.filename, xfile, yfile, wcstr);
+	    }
 	break;
     case XK_F3:
-	if (iswcs (wcs))
+	if (iswcs (wcs)) {
 	    (void)get_cursor_pos (&control.event, &xbuf, &ybuf, &xfile, &yfile);
-	    wcscom (wcs, 3, img.filename, xfile, yfile);
+	    pix2wcst (wcs, xfile, yfile, wcstr, lwcstr);
+	    wcscom (wcs, 3, img.filename, xfile, yfile, wcstr);
+	    }
 	break;
     case XK_F4:
-	if (iswcs (wcs))
+	if (iswcs (wcs)) {
 	    (void)get_cursor_pos (&control.event, &xbuf, &ybuf, &xfile, &yfile);
-	    wcscom (wcs, 4, img.filename, xfile, yfile);
+	    pix2wcst (wcs, xfile, yfile, wcstr, lwcstr);
+	    wcscom (wcs, 4, img.filename, xfile, yfile, wcstr);
+	    }
 	break;
     case XK_F5:
-	if (iswcs (wcs))
+	if (iswcs (wcs)) {
 	    (void)get_cursor_pos (&control.event, &xbuf, &ybuf, &xfile, &yfile);
-	    wcscom (wcs, 5, img.filename, xfile, yfile);
+	    pix2wcst (wcs, xfile, yfile, wcstr, lwcstr);
+	    wcscom (wcs, 5, img.filename, xfile, yfile, wcstr);
+	    }
 	break;
     case XK_F6:
-	if (iswcs (wcs))
+	if (iswcs (wcs)) {
 	    (void)get_cursor_pos (&control.event, &xbuf, &ybuf, &xfile, &yfile);
-	    wcscom (wcs, 6, img.filename, xfile, yfile);
+	    pix2wcst (wcs, xfile, yfile, wcstr, lwcstr);
+	    wcscom (wcs, 6, img.filename, xfile, yfile, wcstr);
+	    }
 	break;
     case XK_F7:
-	if (iswcs (wcs))
+	if (iswcs (wcs)) {
 	    (void)get_cursor_pos (&control.event, &xbuf, &ybuf, &xfile, &yfile);
-	    wcscom (wcs, 7, img.filename, xfile, yfile);
+	    pix2wcst (wcs, xfile, yfile, wcstr, lwcstr);
+	    wcscom (wcs, 7, img.filename, xfile, yfile, wcstr);
+	    }
 	break;
     case XK_F8:
-	if (iswcs (wcs))
+	if (iswcs (wcs)) {
 	    (void)get_cursor_pos (&control.event, &xbuf, &ybuf, &xfile, &yfile);
-	    wcscom (wcs, 8, img.filename, xfile, yfile);
+	    pix2wcst (wcs, xfile, yfile, wcstr, lwcstr);
+	    wcscom (wcs, 8, img.filename, xfile, yfile, wcstr);
+	    }
 	break;
     case XK_F9:
-	if (iswcs (wcs))
+	if (iswcs (wcs)) {
 	    (void)get_cursor_pos (&control.event, &xbuf, &ybuf, &xfile, &yfile);
-	    wcscom (wcs, 9, img.filename, xfile, yfile);
+	    pix2wcst (wcs, xfile, yfile, wcstr, lwcstr);
+	    wcscom (wcs, 9, img.filename, xfile, yfile, wcstr);
+	    }
 	break;
 
     /* Print all cursor commands to standard output */
@@ -740,4 +775,8 @@ PrintFITSHead (header)
  * Nov 30 1998	DJM - Add back region exculsion control
  *
  * Feb 23 1999	DJM - Set face for quad cube projections in +-012345
+ * Aug 16 1999	DJM - Set number of decimal places when toggling with H
+ * Aug 20 1999	DJM - Add WCS string argument to wcscom(); decouple X Y
+ * Aug 20 1999	DJM - Add coordinate return from print_center()
+ * Aug 23 1999	DJM - Fix bug in oldxpix,oldypix saving
  */
