@@ -28,6 +28,8 @@ static char SccsId[] = "%W%  %G%";
  *              {9} Doug Mink Change WCS command initialization  14 Aug 1998
  *             {10} Doug Mink Drop WCS command initialization    29 Sep 1998
  *             {11} Doug Mink Save FITS header in image struct   14 Oct 1998
+ *             {12} Doug Mink Add multiple WCS specified by :c   21 Feb 2001
+ *             {13} Doug Mink Change WCS specification to %c      8 Mar 2001
  *		{n} <who> -- <does what> -- <when>
  */
 
@@ -63,7 +65,7 @@ int init_fits ( img )
   int fd;
   int headlen, nbhead;
   int bitpix, naxes, naxis[3];
-  char *header, *ext;
+  char *header, *ext, *mwcs, cext, *rbrac;
   int lseek_disk();
   struct WorldCoor *wcsinit();
 
@@ -145,18 +147,42 @@ int init_fits ( img )
   else
     img->fscaled = 0;
 
+  /* Check for FITS WCS specification and ignore for file opening */
+    mwcs = strchr (img->filename, '%');
+    if (mwcs != NULL)
+      *mwcs = (char) 0;
+
   /* Set file pointer to start of data */
   ext = strchr (img->filename, ',');
-  if (ext != NULL)
+  if (ext != NULL) {
+    cext = *ext;
     *ext = 0;
+    }
+  else {
+    ext = strchr (img->filename, '[');
+    if (ext != NULL) {
+      cext = *ext;
+      rbrac = strchr (img->filename, ']');
+      if (rbrac != NULL)
+        *rbrac = (char) 0;
+      }
+    }
+
   fd = open_disk(img->filename, IOP_Read, 0);
   if (ext != NULL)
-    *ext = ',';
+    *ext = cext;
+  if (rbrac != NULL)
+    *rbrac = ']';
+  if (mwcs != NULL)
+    *mwcs = '%';
   if (fd <= 0 )
     return( -1 );
 
   /* Initialize WCS structure */
-  wcs = wcsinit (header);
+  if (mwcs != NULL)
+    wcs = wcsinitn (header, mwcs+1);
+  else
+    wcs = wcsinit (header);
   (void)setwcslin (wcs, 1);
   if (control.verbose) {
     if (wcs)
@@ -184,4 +210,7 @@ int init_fits ( img )
  * Aug 14 1998	Use setwcscom() instead of wcscominit()
  * Sep 29 1998	Drop setwcscom() as it is in wcsinit()
  * Oct 14 1998	Save header in image structure
+ *
+ * Feb 21 2001	Implement multiple WCS's specified by :name or :char after filename
+ * Mar  8 2001	Change WCS separator from : to %
  */
